<?php

defined ('_JEXEC') or die('Restricted access');


if (!class_exists ('vmPSPlugin')) {
	require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
}

class plgVmPaymentPayanyway extends vmPSPlugin {

	// Заказ оплачен. Уведомление об оплате магазину доставлено
	const PAW_ORDER_PAID        = 200;
	// Заказ находится в обработке. Точный статус оплаты заказа определить невозможно.
	const PAW_ORDER_WAITING	   = 302;
	// Заказ создан и готов к оплате. Уведомление об оплате магазину не доставлено.
	const PAW_ORDER_ACCEPTED	   = 402;
	// Заказ не является актуальным в магазине (например, заказ отменен).
	const PAW_ORDER_CANCELED	   = 500;

	function __construct (& $subject, $config) {

		parent::__construct ($subject, $config);
		// 		vmdebug('Plugin stuff',$subject, $config);
		$this->_loggable = TRUE;
		$this->tableFields = array_keys ($this->getTableSQLFields ());
		$this->_tablepkey = 'id';
		$this->_tableId = 'id';
		$varsToPush = $this->getVarsToPush ();
		$this->setConfigParameterable ($this->_configTableFieldName, $varsToPush);
	}

	/**
	 * Create the table for this plugin if it does not yet exist.
	 *
	 * @author Valérie Isaksen
	 */
	public function getVmPluginCreateTableSQL () {

		return $this->createTableSQL('Payment Payanyway Table');
	}

	/**
	 * Fields to create the payment table
	 *
	 * @return string SQL Fileds
	 */
	function getTableSQLFields () {

		$SQLfields = array(
			'id'                          => 'int(1) UNSIGNED NOT NULL AUTO_INCREMENT',
			'virtuemart_order_id'         => 'int(1) UNSIGNED',
			'order_number'                => 'char(64)',
			'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED',
			'payment_name'                => 'varchar(5000)',
			'payment_order_total'         => 'decimal(15,5) NOT NULL DEFAULT \'0.00000\'',
			'payment_currency'            => 'char(3)',
			'email_currency'              => 'char(3)',
			'cost_per_transaction'        => 'decimal(10,2)',
			'cost_percent_total'          => 'decimal(10,2)',
			'tax_id'                      => 'smallint(1)'
		);

		return $SQLfields;
	}

	/**
	 *
	 *
	 * @author Valérie Isaksen
	 */
	function plgVmConfirmedOrder ($cart, $order) {

		if (!($method = $this->getVmPluginMethod ($order['details']['BT']->virtuemart_paymentmethod_id))) {
			return NULL; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement ($method->payment_element)) {
			return FALSE;
		}

		vmLanguage::loadJLang('com_virtuemart', true);
		vmLanguage::loadJLang('com_virtuemart_orders', true);

		if (!class_exists ('VirtueMartModelOrders')) {
			require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
		}

		JHTML::_('behavior.formvalidator');
		vmJsApi::addJScript( '/plugins/vmpayment/payanyway/payanyway/assets/js/functions.js');

		$this->getPaymentCurrency($method);
		$currency_code_3 = shopFunctions::getCurrencyByID($method->payment_currency, 'currency_code_3');
		$email_currency = $this->getEmailCurrency($method);
		$totalInPaymentCurrency = vmPSPlugin::getAmountInCurrency($order['details']['BT']->order_total,$method->payment_currency);
		$totalInPaymentCurrency['value'] = number_format($totalInPaymentCurrency['value'], 2, '.', '');

		$payment_method = explode('_',$method->payment_method);
		$unitId = $this->params->get($method->payment_method, '');
		$accountId = isset($payment_method[2]) ? $payment_method[2] : 0;

		$mnt_signature = md5($method->mnt_id.$order['details']['BT']->order_number.$totalInPaymentCurrency['value'].$currency_code_3.$method->mnt_test_mode.$method->mnt_dataintegrity_code);

        $form_action = "https://".$method->mnt_payment_server."/assistant.htm";

		$post_variables = Array(
			'pm' => $order['details']['BT']->virtuemart_paymentmethod_id,
			'MNT_ID' => $method->mnt_id,
			'MNT_TRANSACTION_ID' => $order['details']['BT']->order_number,
			'MNT_CURRENCY_CODE' => $currency_code_3,
			'MNT_AMOUNT' => $totalInPaymentCurrency['value'],
			'MNT_TEST_MODE' => $method->mnt_test_mode,
			'MNT_SIGNATURE' => $mnt_signature,
			'MNT_SUCCESS_URL' => JROUTE::_(JURI::root() . 'index.php?option=com_virtuemart&view=pluginresponse&task=pluginresponsereceived&pm=' . $order['details']['BT']->virtuemart_paymentmethod_id),
			'MNT_FAIL_URL' => JROUTE::_(JURI::root() . 'index.php?option=com_virtuemart&view=pluginresponse&task=pluginUserPaymentCancel&on=' . $order['details']['BT']->order_number . '&pm=' . $order['details']['BT']->virtuemart_paymentmethod_id),
			'payment_method' => $payment_method[0],

		);
		if ($unitId) {
			$post_variables['paymentSystem.unitId'] = $unitId;
			if (strpos($payment_method[0], 'payanyway') !== 0) {
				$post_variables['paymentSystem.limitIds'] = $unitId;
			}
		}
		if ($accountId) {
			$post_variables['paymentSystem.accountId'] = $accountId;
		}
		if (strpos($payment_method[0], 'payanyway') !== 0) {
			$post_variables['followup'] = 'true';
			$post_variables['javascriptEnabled'] = 'true';
		}

		$dbValues['payment_name'] = $this->renderPluginName ($method) . '<br />' . $method->payment_info;
		$dbValues['order_number'] = $order['details']['BT']->order_number;
		$dbValues['virtuemart_paymentmethod_id'] = $order['details']['BT']->virtuemart_paymentmethod_id;
		$dbValues['cost_per_transaction'] = $method->cost_per_transaction;
		$dbValues['cost_percent_total'] = $method->cost_percent_total;
		$dbValues['payment_currency'] = $currency_code_3;
		$dbValues['email_currency'] = $email_currency;
		$dbValues['payment_order_total'] = $totalInPaymentCurrency['value'];
		$dbValues['tax_id'] = $method->tax_id;
		$this->storePSPluginInternalData ($dbValues);

//		$payment_info='';
//		if (!empty($method->payment_info)) {
//			$lang = JFactory::getLanguage ();
//			if ($lang->hasKey ($method->payment_info)) {
//				$payment_info = vmText::_ ($method->payment_info);
//			} else {
//				$payment_info = $method->payment_info;
//			}
//		}
//		if (!class_exists ('VirtueMartModelCurrency')) {
//			require(VMPATH_ADMIN . DS . 'models' . DS . 'currency.php');
//		}
//		$currency = CurrencyDisplay::getInstance ('', $order['details']['BT']->virtuemart_vendor_id);
//
		$html = $this->renderByLayout('post_payment', array(
			'order_number' =>$order['details']['BT']->order_number,
			'order_pass' =>$order['details']['BT']->order_pass,
			'payment_name' => $dbValues['payment_name'],
			'displayTotalInPaymentCurrency' => $totalInPaymentCurrency['display'],
			'payment_action' => $form_action,
			'params' => $post_variables,
			'method' => $method
		));

		$modelOrder = VmModel::getModel ('orders');
		$order['order_status'] = $this->getNewStatus ($method);
		$order['customer_notified'] = 1;
		$order['comments'] = '';
		$modelOrder->updateStatusForOneOrder ($order['details']['BT']->virtuemart_order_id, $order, TRUE);

//		//We delete the old stuff
//		$cart->emptyCart ();
		vRequest::setVar ('html', $html);
		return TRUE;
	}

	/*
		 * Keep backwards compatibility
		 * a new parameter has been added in the xml file
		 */
	function getNewStatus ($method) {

		if (isset($method->status_pending) and $method->status_pending!="") {
			return $method->status_pending;
		} else {
			return 'P';
		}
	}

	/**
	 * Display stored payment data for an order
	 *
	 */
	function plgVmOnShowOrderBEPayment ($virtuemart_order_id, $virtuemart_payment_id) {
		if (!$this->selectedThisByMethodId ($virtuemart_payment_id)) {
			return NULL; // Another method was selected, do nothing
		}

		if (!($paymentTable = $this->getDataByOrderId ($virtuemart_order_id))) {
			return NULL;
		}
		vmLanguage::loadJLang('com_virtuemart');

		$html = '<table class="adminlist table">' . "\n";
		$html .= $this->getHtmlHeaderBE ();
		$html .= $this->getHtmlRowBE ('COM_VIRTUEMART_PAYMENT_NAME', $paymentTable->payment_name);
		$html .= $this->getHtmlRowBE ('STANDARD_PAYMENT_TOTAL_CURRENCY', $paymentTable->payment_order_total . ' ' . $paymentTable->payment_currency);
		if ($paymentTable->email_currency) {
			$html .= $this->getHtmlRowBE ('STANDARD_EMAIL_CURRENCY', $paymentTable->email_currency );
		}
		$html .= '</table>' . "\n";
		return $html;
	}

	/*	function getCosts (VirtueMartCart $cart, $method, $cart_prices) {

			if (preg_match ('/%$/', $method->cost_percent_total)) {
				$cost_percent_total = substr ($method->cost_percent_total, 0, -1);
			} else {
				$cost_percent_total = $method->cost_percent_total;
			}
			return ($method->cost_per_transaction + ($cart_prices['salesPrice'] * $cost_percent_total * 0.01));
		}
	*/


	/**
	 * Check if the payment conditions are fulfilled for this payment method
	 *
	 * @author: Valerie Isaksen
	 *
	 * @param $cart
	 * @param $method
	 * @param $cart_prices
	 * @return true: if the conditions are fulfilled, false otherwise
	 *
	 */
	protected function checkConditions ($cart, $method, $cart_prices) {

		$this->convert_condition_amount($method);
		$amount = $this->getCartAmount($cart_prices);
		$address = (($cart->ST == 0) ? $cart->BT : $cart->ST);


		//vmdebug('standard checkConditions',  $amount, $cart_prices['salesPrice'],  $cart_prices['salesPriceCoupon']);
		$amount_cond = ($amount >= $method->min_amount AND $amount <= $method->max_amount
			OR
			($method->min_amount <= $amount AND ($method->max_amount == 0)));
		if (!$amount_cond) {
			return FALSE;
		}
		$countries = array();
		if (!empty($method->countries)) {
			if (!is_array ($method->countries)) {
				$countries[0] = $method->countries;
			} else {
				$countries = $method->countries;
			}
		}

		// probably did not gave his BT:ST address
		if (!is_array ($address)) {
			$address = array();
			$address['virtuemart_country_id'] = 0;
		}

		if (!isset($address['virtuemart_country_id'])) {
			$address['virtuemart_country_id'] = 0;
		}
		if (count ($countries) == 0 || in_array ($address['virtuemart_country_id'], $countries) ) {
			return TRUE;
		}

		return FALSE;
	}


	/**
	 * displays the logos of a VirtueMart plugin
	 *
	 * @author Valerie Isaksen
	 * @author Max Milbers
	 * @param array $logo_list
	 * @return html with logos
	 */
	protected function displayLogos ($logo_list) {

		$img = "";
		if (!(empty($logo_list))) {
			$url = JURI::root () . 'plugins/vmpayment/payanyway/payanyway/assets/images/systems/';
			if (!is_array ($logo_list)) {
				$logo_list = (array)$logo_list;
			}
			foreach ($logo_list as $logo) {
				$alt_text = substr ($logo, 0, strpos ($logo, '.'));
				$img .= '<span class="vmCartPaymentLogo" ><img align="middle" src="' . $url . $logo . '"  alt="' . $alt_text . '" /></span> ';
			}
		}
		return $img;
	}

	/*
	* We must reimplement this triggers for joomla 1.7
	*/

	/**
	 * Create the table for this plugin if it does not yet exist.
	 * This functions checks if the called plugin is active one.
	 * When yes it is calling the standard method to create the tables
	 *
	 * @author Valérie Isaksen
	 *
	 */
	function plgVmOnStoreInstallPaymentPluginTable ($jplugin_id) {

		return $this->onStoreInstallPluginTable ($jplugin_id);
	}

	/**
	 * This event is fired after the payment method has been selected. It can be used to store
	 * additional payment info in the cart.
	 *
	 * @author Max Milbers
	 * @author Valérie isaksen
	 *
	 * @param VirtueMartCart $cart: the actual cart
	 * @param $msg
	 * @return null if the payment was not selected, true if the data is valid, error message if the data is not vlaid
	 *
	 */
	public function plgVmOnSelectCheckPayment (VirtueMartCart $cart, &$msg) {

		return $this->OnSelectCheck ($cart);
	}

	/**
	 * plgVmDisplayListFEPayment
	 * This event is fired to display the pluginmethods in the cart (edit shipment/payment) for exampel
	 *
	 * @param VirtueMartCart  $cart Cart object
	 * @param integer $selected ID of the method selected
	 * @param $htmlIn
	 * @return boolean True on succes, false on failures, null when this plugin was not selected.
	 * On errors, JError::raiseWarning (or JError::raiseError) must be used to set a message.
	 *
	 * @author Valerie Isaksen
	 * @author Max Milbers
	 */
	public function plgVmDisplayListFEPayment (VirtueMartCart $cart, $selected = 0, &$htmlIn) {

		return $this->displayListFE ($cart, $selected, $htmlIn);
	}

	/*
	* plgVmonSelectedCalculatePricePayment
	* Calculate the price (value, tax_id) of the selected method
	* It is called by the calculator
	* This function does NOT to be reimplemented. If not reimplemented, then the default values from this function are taken.
	* @author Valerie Isaksen
	* @cart: VirtueMartCart the current cart
	* @cart_prices: array the new cart prices
	* @return null if the method was not selected, false if the shiiping rate is not valid any more, true otherwise
	*
	*
	*/

	public function plgVmonSelectedCalculatePricePayment (VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {

		return $this->onSelectedCalculatePrice ($cart, $cart_prices, $cart_prices_name);
	}

	function plgVmgetPaymentCurrency ($virtuemart_paymentmethod_id, &$paymentCurrencyId) {

		if (!($method = $this->getVmPluginMethod ($virtuemart_paymentmethod_id))) {
			return NULL; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement ($method->payment_element)) {
			return FALSE;
		}
		$this->getPaymentCurrency ($method);

		$paymentCurrencyId = $method->payment_currency;
		return;
	}

	/**
	 * plgVmOnCheckAutomaticSelectedPayment
	 * Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
	 * The plugin must check first if it is the correct type
	 *
	 * @author Valerie Isaksen
	 * @param VirtueMartCart cart: the cart object
	 * @param array $cart_prices
	 * @param $paymentCounter
	 * @return null if no plugin was found, 0 if more then one plugin was found,  virtuemart_xxx_id if only one plugin is found
	 *
	 */
	function plgVmOnCheckAutomaticSelectedPayment (VirtueMartCart $cart, array $cart_prices = array(), &$paymentCounter) {

		return $this->onCheckAutomaticSelected ($cart, $cart_prices, $paymentCounter);
	}

	/**
	 * This method is fired when showing the order details in the frontend.
	 * It displays the method-specific data.
	 *
	 * @param integer $virtuemart_order_id The order ID
	 * @param integer $virtuemart_paymentmethod_id
	 * @param string $payment_name
	 * @return mixed Null for methods that aren't active, text (HTML) otherwise
	 * @author Max Milbers
	 * @author Valerie Isaksen
	 */
	public function plgVmOnShowOrderFEPayment ($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name) {

		$this->onShowOrderFE ($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
	}


	/**
	 * @param $virtuemart_paymentmethod_id
	 * @param $virtuemart_order_id
	 * @param $emailCurrencyId
	 * @return bool|null
	 */
	function plgVmgetEmailCurrency($virtuemart_paymentmethod_id, $virtuemart_order_id, &$emailCurrencyId) {

		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
			return NULL; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
			return FALSE;
		}
		if (!($payments = $this->getDatasByOrderId($virtuemart_order_id))) {
			// JError::raiseWarning(500, $db->getErrorMsg());
			return '';
		}
		if (empty($payments[0]->email_currency)) {
			$vendorId = 1; //VirtueMartModelVendor::getLoggedVendor();
			$db = JFactory::getDBO();
			$q = 'SELECT   `vendor_currency` FROM `#__virtuemart_vendors` WHERE `virtuemart_vendor_id`=' . $vendorId;
			$db->setQuery($q);
			$emailCurrencyId = $db->loadResult();
		} else {
			$emailCurrencyId = $payments[0]->email_currency;
		}

	}

	/**
	 * This event is fired during the checkout process. It can be used to validate the
	 * method data as entered by the user.
	 *
	 * @return boolean True when the data was valid, false otherwise. If the plugin is not activated, it should return null.
	 * @author Max Milbers

	public function plgVmOnCheckoutCheckDataPayment(  VirtueMartCart $cart) {
	return null;
	}
	 */

	/**
	 * This method is fired when showing when priting an Order
	 * It displays the the payment method-specific data.
	 *
	 * @param integer $order_number The order ID
	 * @param integer $method_id  method used for this order
	 * @return mixed Null when for payment methods that were not selected, text (HTML) otherwise
	 * @author Valerie Isaksen
	 */
	function plgVmonShowOrderPrintPayment ($order_number, $method_id) {

		return $this->onShowOrderPrint ($order_number, $method_id);
	}

	function plgVmDeclarePluginParamsPaymentVM3( &$data) {
		return $this->declarePluginParams('payment', $data);
	}
	function plgVmSetOnTablePluginParamsPayment ($name, $id, &$table) {

		return $this->setOnTablePluginParams ($name, $id, $table);
	}

	//Notice: We only need to add the events, which should work for the specific plugin, when an event is doing nothing, it should not be added

	/**
	 * Save updated order data to the method specific table
	 *
	 * @param string $html : the html to display
	 * @internal param array $_formData Form data
	 * @return mixed, True on success, false on failures (the rest of the save-process will be
	 *                     skipped!), or null when this method is not actived.
	 *
	 * public function plgVmOnUpdateOrderPayment(  $_formData) {
	 * return null;
	 * }
	 *
	 * /**
	 * Save updated orderline data to the method specific table
	 *
	 * @internal param array $_formData Form data
	 * @return mixed, True on success, false on failures (the rest of the save-process will be
	 * skipped!), or null when this method is not actived.
	 *
	 * public function plgVmOnUpdateOrderLine(  $_formData) {
	 * return null;
	 * }
	 *
	 * /**
	 * plgVmOnEditOrderLineBE
	 * This method is fired when editing the order line details in the backend.
	 * It can be used to add line specific package codes
	 *
	 * @internal param int $_orderId The order ID
	 * @internal param int $_lineId
	 * @return mixed Null for method that aren't active, text (HTML) otherwise
	 *
	 * public function plgVmOnEditOrderLineBEPayment(  $_orderId, $_lineId) {
	 * return null;
	 * }
	 *
	 * /**
	 * This method is fired when showing the order details in the frontend, for every orderline.
	 * It can be used to display line specific package codes, e.g. with a link to external tracking and
	 * tracing systems
	 *
	 * @internal param int $_orderId The order ID
	 * @internal param int $_lineId
	 * @return mixed Null for method that aren't active, text (HTML) otherwise
	 *
	 * public function plgVmOnShowOrderLineFE(  $_orderId, $_lineId) {
	 * return null;
	 * }
	 *
	 * /**
	 * This event is fired when the  method notifies you when an event occurs that affects the order.
	 * Typically,  the events  represents for payment authorizations, Fraud Management Filter actions and other actions,
	 * such as refunds, disputes, and chargebacks.
	 *
	 * NOTE for Plugin developers:
	 *  If the plugin is NOT actually executed (not the selected payment method), this method must return NULL
	 *
	 * @internal param $return_context : it was given and sent in the payment form. The notification should return it back.
	 * Used to know which cart should be emptied, in case it is still in the session.
	 * @internal param int $virtuemart_order_id : payment  order id
	 * @internal param \char $new_status : new_status for this order id.
	 * @return mixed Null when this method was not selected, otherwise the true or false
	 *
	 * @author   Valerie Isaksen
	 *
	 */
	 public function plgVmOnPaymentNotification() {
	    return null;
	 }


	// варианты налогообложения:
	const paw_kassa_VAT0     = 1104;  // НДС 0%
	const paw_kassa_VAT10    = 1103;  // НДС 10%
	const paw_kassa_VAT20    = 1102;  // НДС 20%
	const paw_kassa_VATNOVAT = 1105;  // НДС не облагается
	const paw_kassa_VATWR10  = 1107;  // НДС с рассч. ставкой 10%
	const paw_kassa_VATWR20  = 1106;  // НДС с рассч. ставкой 20%

	private static function monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, $resultCode = '500',
												 $echo = true, $die = true, $kassa_inventory = null, $kassa_customer = null, $kassa_delivery = null)
	{
		$mnt_signature = md5($resultCode.$mnt_id.$mnt_transaction_id.$mnt_data_integrity_code);
		$response = '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
		$response .= "<MNT_RESPONSE>\n";
		$response .= "<MNT_ID>{$mnt_id}</MNT_ID>\n";
		$response .= "<MNT_TRANSACTION_ID>{$mnt_transaction_id}</MNT_TRANSACTION_ID>\n";
		$response .= "<MNT_RESULT_CODE>{$resultCode}</MNT_RESULT_CODE>\n";
		$response .= "<MNT_SIGNATURE>{$mnt_signature}</MNT_SIGNATURE>\n";
		if (!empty($kassa_inventory) || !empty($kassa_customer) || !empty($kassa_delivery))
		{
			$response .= "<MNT_ATTRIBUTES>\n";
			foreach (array('INVENTORY' => $kassa_inventory, 'CUSTOMER' => $kassa_customer, 'DELIVERY' => $kassa_delivery) as $k => $v)
				if (!empty($v))
					$response .= "<ATTRIBUTE><KEY>{$k}</KEY><VALUE>{$v}</VALUE></ATTRIBUTE>\n";
			$response .= "</MNT_ATTRIBUTES>\n";
		}
		$response .= "</MNT_RESPONSE>\n";
		if ($echo === true)
		{
			header("Content-type: application/xml");
			echo $response;
		}
		else
			return $response;
		if ($die === true)
			die;
		return '';
	}

	/**
	 * plgVmOnPaymentResponseReceived
	 * This event is fired when the  method returns to the shop after the transaction
	 *
	 *  the method itself should send in the URL the parameters needed
	 * NOTE for Plugin developers:
	 *  If the plugin is NOT actually executed (not the selected payment method), this method must return NULL
	 *
	 * @param $html
	 * @return mixed Null when this method was not selected, otherwise the true or false
	 *
	 * @author   Valerie Isaksen
	 */
	public function plgVmOnPaymentResponseReceived(&$html) {
//		var_Dump($_REQUEST); die;
//		die('plgVmOnPaymentResponseReceived');
		// the payment itself should send the parameter needed.
		$input = \Joomla\CMS\Factory::getApplication()->input;
		$virtuemart_paymentmethod_id = $input -> getInt('pm', 0);

		$vendorId = 0;
		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
			return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
			return false;
		}


		// $payment_data = JRequest::get();
		$payment_data = $input -> getArray();

		$order_number = $payment_data['MNT_TRANSACTION_ID'];
		// $order_number = $input -> get('MNT_TRANSACTION_ID');
		if (!class_exists('VirtueMartModelOrders'))
			require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );

		$virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number);
		$payment_name = $this->renderPluginName($method);

		$modelOrder = new VirtueMartModelOrders();
		$order = $modelOrder->getOrder($virtuemart_order_id);

		switch ($payment_data['action']) {
			case 'callback':
				$kassaInventory = '';
				$kassaDelivery = '';
				$client_email = '';
				$updateStatus = false;
				$responseCode = self::PAW_ORDER_CANCELED;

				if (isset($payment_data['MNT_ID']) && isset($payment_data['MNT_TRANSACTION_ID']) && isset($payment_data['MNT_AMOUNT']) && isset($payment_data['MNT_CURRENCY_CODE']) && isset($payment_data['MNT_TEST_MODE']) && isset($payment_data['MNT_SIGNATURE']))  {
					if ($this->_checkSignature($payment_data, $method)){
						$amount = (float) $payment_data['MNT_AMOUNT'];
						$this->getPaymentCurrency($method);
						$totalInPaymentCurrency = vmPSPlugin::getAmountInCurrency($order['details']['BT']->order_total,$method->payment_currency);
						$totalInPaymentCurrency['value'] = number_format($totalInPaymentCurrency['value'], 2, '.', '');
						if ( (!isset($payment_data['MNT_COMMAND']) or $payment_data['MNT_COMMAND'] != 'CHECK') && ($totalInPaymentCurrency['value'] == $amount) ) {
							$order['order_status'] = $method->status_paid;
							$order['customer_notified'] = 1;
							$order['comments'] = JText::sprintf('VMPAYMENT_PAYANYWAY_PAYMENT_CONFIRMED', $order_number);
							$responseCode = self::PAW_ORDER_PAID;
							$updateStatus = true;
							// kassa.payanyway.ru:
							{
								$client_email = $order['details']['BT']->email;
								$inventoryPositions = array();
								$kassaAmount = 0;
								foreach ($order['items'] as $item)
								{
									$vatTag = self::paw_kassa_VATNOVAT;
									$itemId = $item->virtuemart_order_item_id;
									foreach($order["calc_rules"] as $tax) {
										if ($tax->virtuemart_order_item_id == $itemId) {
											break;
										}
									}
									if ($tax->calc_mathop == "+%")
									{
										$item_tax = floatval($tax->calc_value);
										if ($item_tax == 10)
											$vatTag = self::paw_kassa_VATWR10;
										if ($item_tax == 20)
											$vatTag = self::paw_kassa_VATWR20;
									}
									$price = number_format(floor(floatval($item->product_final_price) * 100) * 0.01, 2, '.', '');
									$inventoryPositions[] = array(
										'name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($item->product_name))),
										'price' => $price,
										'quantity' => $item->product_quantity,
										'vatTag' => $vatTag
									);
									$kassaAmount += floatval($price * $item->product_quantity);
								}
								$costOfDelivery = floor((floatval($order['details']['BT']->order_shipment) + floatval($order['details']['BT']->order_shipment_tax)) * 100) * 0.01;
								$shippingVATTag = self::paw_kassa_VATNOVAT;
								foreach ($order['calc_rules'] as $rule)
								{
									if ($rule->calc_kind == 'shipment' && $rule->calc_mathop == '+%')
									{
										if ($rule->calc_value == 10)
											$shippingVATTag = self::paw_kassa_VATWR10;
										if ($rule->calc_value == 20)
											$shippingVATTag = self::paw_kassa_VATWR20;
									}
								}
								if ($costOfDelivery > 0)
									$kassaDelivery = $costOfDelivery;
								$kassaInventory = json_encode($inventoryPositions);
							}
						} else
						{
							if ($order->order_status == $method->status_paid)
								$responseCode = self::PAW_ORDER_PAID;
							else if ($order->order_status == $method->status_canceled)
								$responseCode = self::PAW_ORDER_CANCELED;
							else if ($order->order_status == $method->status_pending)
								$responseCode = self::PAW_ORDER_ACCEPTED;
							else
								$responseCode = self::PAW_ORDER_WAITING;
						}
					}
				}
				self::monetaPayURLResponse($payment_data['MNT_ID'], $payment_data['MNT_TRANSACTION_ID'],
					$method->mnt_dataintegrity_code, $responseCode, true, false, $kassaInventory, $client_email, $kassaDelivery);

				if ($updateStatus)
					$modelOrder->updateStatusForOneOrder($virtuemart_order_id, $order, true);

				exit;
				break;
			default:
				$order = $this->getDataByOrderId($virtuemart_order_id);
				$html = $this->_getPaymentResponseHtml($payment_data, $payment_name, $order);

				if ($virtuemart_order_id) {
					if (!class_exists('VirtueMartCart'))
						require(JPATH_VM_SITE . DS . 'helpers' . DS . 'cart.php');
					// get the correct cart / session
					$cart = VirtueMartCart::getCart();

					$cart->emptyCart();
				}
				break;
		}
		return true;
	}


	private function _checkSignature($payment_data, $method) {
		$params = '';
		if (isset($payment_data['MNT_COMMAND'])) $params .= $payment_data['MNT_COMMAND'];
		$params .= $payment_data['MNT_ID'] . $payment_data['MNT_TRANSACTION_ID'];
		if (isset($payment_data['MNT_OPERATION_ID'])) $params .= $payment_data['MNT_OPERATION_ID'];
		if (isset($payment_data['MNT_AMOUNT'])) $params .= $payment_data['MNT_AMOUNT'];
		$params .= $payment_data['MNT_CURRENCY_CODE'];
		if (isset($payment_data['MNT_SUBSCRIBER_ID'])) $params .= $payment_data['MNT_SUBSCRIBER_ID'];
		$params .= $payment_data['MNT_TEST_MODE'];
		$signature = md5($params . $method->mnt_dataintegrity_code);
		if(strcasecmp($signature, $payment_data['MNT_SIGNATURE'] ) == 0) {
			return true;
		}
		return false;
	}

	public function _getXMLResponse($payment_data, $method, $resultCode)
	{
		$signature = md5($resultCode . $payment_data['MNT_ID'] . $payment_data['MNT_TRANSACTION_ID'] . $method->mnt_dataintegrity_code);
		$result = '<?xml version="1.0" encoding="UTF-8" ?>';
		$result .= '<MNT_RESPONSE>';
		$result .= '<MNT_ID>' . $payment_data['MNT_ID'] . '</MNT_ID>';
		$result .= '<MNT_TRANSACTION_ID>' . $payment_data['MNT_TRANSACTION_ID'] . '</MNT_TRANSACTION_ID>';
		$result .= '<MNT_RESULT_CODE>' . $resultCode . '</MNT_RESULT_CODE>';
		$result .= '<MNT_SIGNATURE>' . $signature . '</MNT_SIGNATURE>';
		$result .= '</MNT_RESPONSE>';
		return $result;
	}

	private function _getPaymentResponseHtml($payment_data, $payment_name, $order)
	{
		vmdebug('payanyway response', $payment_data);

		$html = '<table>' . "\n";
		$html .= $this->getHtmlRow('PAYANYWAY_PAYMENT_NAME', $order->payment_name);
		$html .= $this->getHtmlRow('PAYANYWAY_ORDER_NUMBER', $order->order_number);
		$html .= $this->getHtmlRow('PAYANYWAY_AMOUNT', number_format($order->payment_order_total,2,'.','')  . " " . $order->payment_currency);

		$html .= '</table>' . "\n";

		return $html;
	}
}
